# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


DE_MONTHS = [
    "Januar",
    "Februar",
    "März",
    "April",
    "Mai",
    "Juni",
    "Juli",
    "August",
    "September",
    "Oktober",
    "November",
    "Dezember",
]
DE_MONTHS_LC = [
    "januar",
    "februar",
    "märz",
    "april",
    "mai",
    "juni",
    "juli",
    "august",
    "september",
    "oktober",
    "november",
    "dezember",
]

# German months map
de_mname2mon = dict((m, i + 1) for i, m in enumerate(DE_MONTHS) if m)
delc_mname2mon = dict((m, i + 1) for i, m in enumerate(DE_MONTHS_LC) if m)

BASE_PATTERNS_DE = {
    "pat:de:months": oneOf(DE_MONTHS).setParseAction(lambda t: de_mname2mon[t[0]]),
    "pat:de:months_lc": oneOf(DE_MONTHS_LC).setParseAction(
        lambda t: delc_mname2mon[t[0]]
    ),
}

PATTERNS_DE = [
    # German patterns
    {
        "key": "dt:date:de_base",
        "name": "Base german date with month name",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_DE["pat:de:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:de_base_lc",
        "name": "Base german date with month name and lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_DE["pat:de:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


BG_MONTHS = [
    "Янyapи",
    "Фeвpyapи",
    "Мapт",
    "Апpил",
    "Май",
    "Юни",
    "Юли",
    "Авгycт",
    "Сeптeмвpи",
    "Октoмвpи",
    "Нoeмвpи",
    "Дeкeмвpи",
]
BG_MONTHS_LC = [
    "янyapи",
    "фeвpyapи",
    "мapт",
    "aпpил",
    "май",
    "юни",
    "юли",
    "aвгycт",
    "ceптeмвpи",
    "oктoмвpи",
    "нoeмвpи",
    "дeкeмвpи",
]

# Bulgarian months map
bg_mname2mon = dict((m, i + 1) for i, m in enumerate(BG_MONTHS) if m)
bglc_mname2mon = dict((m, i + 1) for i, m in enumerate(BG_MONTHS_LC) if m)

BASE_PATTERNS_BG = {
    "pat:bg:months": oneOf(BG_MONTHS).setParseAction(lambda t: bg_mname2mon[t[0]]),
    "pat:bg:months_lc": oneOf(BG_MONTHS_LC).setParseAction(
        lambda t: bglc_mname2mon[t[0]]
    ),
}

PATTERNS_BG = [
    # Bulgarian patterns
    {
        "key": "dt:date:bg_base",
        "name": "Base bulgarian date with month name",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_BG["pat:bg:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:bg_base_lc",
        "name": "Base bulgarian date with month name and lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_BG["pat:bg:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)
from .base import BASE_DATE_PATTERNS

RUS_MONTHS_ORIG = [
    u"Январь",
    u"Февраль",
    u"Март",
    u"Апрель",
    u"Май",
    u"Июнь",
    u"Июль",
    u"Август",
    u"Сентябрь",
    u"Октябрь",
    u"Ноябрь",
    u"Декабрь",
]
RUS_MONTHS_ORIG_LC = [
    u"январь",
    u"февраль",
    u"март",
    u"апрель",
    u"май",
    u"июнь",
    u"июль",
    u"август",
    u"сентябрь",
    u"октябрь",
    u"ноябрь",
    u"декабрь",
]
RUS_MONTHS = [
    u"Января",
    u"Февраля",
    u"Марта",
    u"Апреля",
    u"Мая",
    u"Июня",
    u"Июля",
    u"Августа",
    u"Сентября",
    u"Октября",
    u"Ноября",
    u"Декабря",
]
RUS_MONTHS_LC = [
    u"января",
    u"февраля",
    u"марта",
    u"апреля",
    u"мая",
    u"июня",
    u"июля",
    u"августа",
    u"сентября",
    u"октября",
    u"ноября",
    u"декабря",
]
RUS_WEEKDAYS = [
    u"Понедельник",
    u"Вторник",
    u"Среда",
    u"Четверг",
    u"Пятница",
    u"Суббота",
    u"Воскресение",
]
RUS_WEEKDAYS_LC = [
    u"понедельник",
    u"вторник",
    u"среда",
    u"четверг",
    u"пятница",
    u"суббота",
    u"воскресение",
]
RUS_YEARS = [u"г.", u"года"]


# Russian months map
ru_mname2mon = dict((m, i + 1) for i, m in enumerate(RUS_MONTHS) if m)
rulc_mname2mon = dict((m, i + 1) for i, m in enumerate(RUS_MONTHS_LC) if m)
ru_origmname2mon = dict((m, i + 1) for i, m in enumerate(RUS_MONTHS_ORIG) if m)
rulc_origmname2mon = dict((m, i + 1) for i, m in enumerate(RUS_MONTHS_ORIG_LC) if m)

BASE_PATTERNS_RU = {
    "pat:rus:years": oneOf(RUS_YEARS),
    "pat:rus:weekdays": oneOf(RUS_WEEKDAYS),
    "pat:rus:weekdays_lc": oneOf(RUS_WEEKDAYS_LC),
    #  months names
    "pat:rus:months": oneOf(RUS_MONTHS).setParseAction(lambda t: ru_mname2mon[t[0]]),
    "pat:rus:months:lc": oneOf(RUS_MONTHS_LC).setParseAction(
        lambda t: rulc_mname2mon[t[0]]
    ),
    # Original months names, very rarely in use
    "pat:rus:monthsorig": oneOf(RUS_MONTHS_ORIG).setParseAction(
        lambda t: ru_origmname2mon[t[0]]
    ),
    "pat:rus:monthsorig:lc": oneOf(RUS_MONTHS_ORIG_LC).setParseAction(
        lambda t: rulc_origmname2mon[t[0]]
    ),
}

PATTERNS_RU = [
    # Russian patterns
    {
        "key": "dt:date:date_rus",
        "name": "Date with russian month",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + BASE_PATTERNS_RU["pat:rus:months"].setResultsName("month")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 20},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:date_rus2",
        "name": "Date with russian month and year word",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + BASE_PATTERNS_RU["pat:rus:months"].setResultsName("month")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year")
        + Optional(BASE_PATTERNS_RU["pat:rus:years"]).suppress(),
        "length": {"min": 13, "max": 20},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:date_rus3",
        "name": "Date with russian year",
        "pattern": BASE_DATE_PATTERNS["pat:date:d.m.yyyy"]
        + BASE_PATTERNS_RU["pat:rus:years"].suppress(),
        "length": {"min": 14, "max": 20},
        "format": "%d.%m.%Y",
    },
    {
        "key": "dt:date:date_rus_lc1",
        "name": "Date with russian month",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + BASE_PATTERNS_RU["pat:rus:months:lc"].setResultsName("month")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:date_rus_lc2",
        "name": "Date with russian month with year word",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + BASE_PATTERNS_RU["pat:rus:months:lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year")
        + Optional(BASE_PATTERNS_RU["pat:rus:years"]).suppress(),
        "length": {"min": 13, "max": 25},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_rus",
        "name": "Date with russian month and weekday",
        "pattern": BASE_PATTERNS_RU["pat:rus:weekdays"]
        + Optional(",")
        + Word(nums, min=1, max=2)
        + BASE_PATTERNS_RU["pat:rus:months"]
        + Optional(Literal(",")).suppress()
        + Word(nums, exact=4).setResultsName("year")
        + BASE_PATTERNS_RU["pat:rus:years"].suppress(),
        "length": {"min": 13, "max": 20},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_rus_lc1",
        "name": "Date with russian month and weekday",
        "pattern": BASE_PATTERNS_RU["pat:rus:weekdays"]
        + Optional(",")
        + Word(nums, min=1, max=2)
        + BASE_PATTERNS_RU["pat:rus:months:lc"]
        + Optional(Literal(",")).suppress()
        + Word(nums, exact=4).setResultsName("year")
        + BASE_PATTERNS_RU["pat:rus:years"].suppress(),
        "length": {"min": 13, "max": 25},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:rus_rare_2",
        "name": "Date with russian month with dots as divider",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_RU["pat:rus:months"].setResultsName("month")
        + Optional(".").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 20},
        "format": "%d.%m.%Y",
        "filter": 1,
    },
    {
        "key": "dt:date:rus_rare_3",
        "name": "Date with russian month with dots as divider with low case months",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Literal(".").suppress()
        + BASE_PATTERNS_RU["pat:rus:months:lc"].setResultsName("month")
        + Literal(".").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 20},
        "format": "%d.%m.%Y",
        "filter": 1,
    },
    # KHMB Bank http://www.kbhmb.ru/news/
    {
        "key": "dt:date:rus_rare_5",
        "name": "Russian date stars with month name",
        "pattern": BASE_PATTERNS_RU["pat:rus:monthsorig"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 13, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    # Bank Rus format http://www.bankrus.ru/about/info/g1/news
    {
        "key": "dt:date:rus_rare_6",
        "name": "Russian date stars with weekday and follows with month name",
        "pattern": BASE_PATTERNS_RU["pat:rus:weekdays_lc"].suppress()
        + Literal(",").suppress()
        + BASE_PATTERNS_RU["pat:rus:months:lc"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 13, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Here is all dirty matched code
__author__ = "Ivan Begtin (ivan@begtin.tech)"
__license__ = "BSD"


def matchPrefix(text):
    """
    This is silver bullet, cornerstone and magic wand of speed of this algorithm
    it filters patterns using manually selected rules. Yes, yes, it's "dirty" code and it could be beautified in
    many ways but this library intended to be fast, not beautiful. Without matching is x1.3 slower so let it be.
    :param text: text with date to match
    :return: list of patterns to run against
    """
    basekeys = []
    if not text[0].isdigit():
        fc = text[0].lower()
        fc = fc if len(fc) > 1 else text[0]
        if fc.isalpha() and ord(fc) in range(ord("a"), ord("z") + 1):
            basekeys = [
                "dt:date:eng1",
                "dt:date:date_eng1x",
                "dt:date:eng3",
                "dt:date:date_eng2_lc",
                "dt:date:date_eng2",
                "dt:date:date_eng2_short",
                "dt:date:date_eng3",
                "dt:date:weekday_eng",
                "dt:date:weekday_eng_lc",
                "dt:date:weekday_eng_wshort",
                "dt:date:weekday_eng_iso",
                "dt:date:weekday_short_eng_iso",
                "dt:date:fr_base_article",
                "dt:date:fr_base_lc_article",
                "dt:date:weekday_eng_mshort_wshort",
            ]
            basekeys += [
                "dt:date:pt_base",
                "dt:date:pt_base_lc",
                "dt:date:pt_base_article",
                "dt:date:pt_base_lc_article",
            ]
            basekeys += [
                "dt:date:es_base",
                "dt:date:es_base_lc",
                "dt:date:es_base_article",
                "dt:date:es_base_lc_article",
                "dt:date:es_rare_1",
                "dt:date:es_rare_2",
            ]
            basekeys += [
                "dt:date:it_base",
                "dt:date:it_base_lc",
                "dt:date:it_base_article",
                "dt:date:it_base_lc_article",
                "dt:date:it_rare_1",
                "dt:date:it_rare_2",
            ]
        else:
            basekeys = [
                "dt:date:weekday_rus",
                "dt:date:weekday_rus_lc1",
                "dt:date:rare_5",
                "dt:date:rare_6",
            ]
    else:
        if text[1] == "." or text[2] == ".":
            basekeys = [
                "dt:date:date_2",
                "dt:date:date_4",
                "dt:date:date_rus3",
                "dt:date:date_4_point",
                "dt:date:date_eng1",
                "dt:date:noyear_1",
                "dt:date:rare_2",
                "dt:date:rare_3",
            ]
            basekeys += ["dt:date:de_base", "dt:date:de_base_lc"]
        elif text[1] == "," or text[2] == ",":
            basekeys = ["dt:date:date_rus"]
        elif text[1] == "/" or text[2] == "/":
            basekeys = [
                "dt:date:date_1",
                "dt:date:date_9",
                "dt:date:date_8",
                "dt:date:date_usa",
                "dt:date:date_usa_1",
                "dt:date:rare_1",
            ]
        elif text[1] == "-" or text[2] == "-":
            basekeys = ["dt:date:date_iso8601", "dt:date:date_iso8601_short"]
        elif text[4] == "-":
            basekeys = ["dt:date:date_iso8601", "dt:date:date_9"]
        elif text[4] == ".":
            basekeys = ["dt:date:date_10"]
        else:
            basekeys = [
                "dt:date:date_3",
                "dt:date:date_5",
                "dt:date:date_6",
                "dt:date:date_7",
                "dt:date:date_rus",
                "dt:date:date_rus2",
                "dt:date:date_rus_lc1",
                "dt:date:date_rus_lc2",
                "dt:date:date_eng1",
                "dt:date:date_eng1_short",
                "dt:date:date_eng1_lc",
                "dt:date:date_eng1xx",
            ]
            # I have to add others date keys since sometimes spaces used in date inside
            basekeys += [
                "dt:date:date_1",
                "dt:date:date_9",
                "dt:date:date_8",
                "dt:date:date_usa",
                "dt:date:date_usa_1",
                "dt:date:rare_1",
                "dt:date:rare_2",
                "dt:date:rare_3",
                "dt:date:rare_4",
                "dt:date:date_5",
                "dt:date:fr_base",
                "dt:date:fr_base_lc",
            ]
            # Adding portugal dates
            basekeys += [
                "dt:date:pt_base_article",
                "dt:date:pt_base_lc_article",
                "dt:date:pt_base",
                "dt:date:pt_base_lc",
            ]
            # Adding german dates
            basekeys += ["dt:date:de_base", "dt:date:de_base_lc"]
            # Adding bulgarian dates
            basekeys += ["dt:date:bg_base", "dt:date:bg_base_lc"]
            # Adding spanish dates
            basekeys += [
                "dt:date:es_base",
                "dt:date:es_base_lc",
                "dt:date:es_base_article",
                "dt:date:es_base_lc_article",
            ]
            basekeys += [
                "dt:date:it_base",
                "dt:date:it_base_lc",
                "dt:date:it_base_article",
                "dt:date:it_base_lc_article",
            ]
    #        print('Basekeys', basekeys, 'for', text)
    return basekeys



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)

# Incomplete

CZ_WEEKDAYS = ["Pondělí", "Úterý", "Středa", "Čtvrtek", "Pátek", "Sobota", "Neděle"]
CZ_WEEKDAYS_LC = ["pondělí", "úterý", "středa", "čtvrtek", "pátek", "sobota", "neděle"]
CZ_MONTHS = [
    "Leden",
    "Únor",
    "Březen",
    "Duben",
    "Květen",
    "Červen",
    "Červenec",
    "Srpen",
    "Září",
    "Říjen",
    "Listopad",
    "Prosinec",
]
CZ_MONTHS_LC = [
    "leden",
    "únor",
    "březen",
    "duben",
    "květen",
    "červen",
    "červenec",
    "srpen",
    "září",
    "říjen",
    "listopad",
    "prosinec",
]

# Czech months map
cz_mname2mon = dict((m, i + 1) for i, m in enumerate(CZ_MONTHS) if m)
czlc_mname2mon = dict((m, i + 1) for i, m in enumerate(CZ_MONTHS_LC) if m)

BASE_PATTERNS_CZ = {
    "pat:cz:weekdays": oneOf(CZ_WEEKDAYS),
    "pat:cz:weekdays_lc": oneOf(CZ_WEEKDAYS_LC),
    "pat:cz:months": oneOf(CZ_MONTHS).setParseAction(lambda t: cz_mname2mon[t[0]]),
    "pat:cz:months_lc": oneOf(CZ_MONTHS_LC).setParseAction(
        lambda t: czlc_mname2mon[t[0]]
    ),
}

PATTERNS_CZ = []



# -*- coding: utf-8 -*-
__version__ = "0.1.1"
__author__ = "Ivan Begtin (ivan@begtin.tech)"
__license__ = "BSD"

from .bg import PATTERNS_BG
from .cz import PATTERNS_CZ
from .de import PATTERNS_DE
from .es import PATTERNS_ES
from .fr import PATTERNS_FR
from .it import PATTERNS_IT
from .pt import PATTERNS_PT
from .ru import PATTERNS_RU
from .base import PATTERNS_EN, BASE_TIME_PATTERNS, INTEGER_LIKE_PATTERNS

ALL_PATTERNS = (
    PATTERNS_EN
    + INTEGER_LIKE_PATTERNS
    + PATTERNS_BG
    + PATTERNS_CZ
    + PATTERNS_DE
    + PATTERNS_ES
    + PATTERNS_FR
    + PATTERNS_IT
    + PATTERNS_PT
    + PATTERNS_RU
)

SUPPORTED_LANGUAGES = ["bg", "cz", "de", "en", "fr", "it", "pt", "ru"]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


FR_MONTHS = [
    "Janvier",
    "Février",
    "Mars",
    "Avril",
    "Mai",
    "Juin",
    "Juillet",
    "Août",
    "Septembre",
    "Octobre",
    "Novembre",
    "Décembre",
]
FR_MONTHS_LC = [
    "janvier",
    "février",
    "mars",
    "avril",
    "mai",
    "juin",
    "juillet",
    "août",
    "septembre",
    "octobre",
    "novembre",
    "décembre",
]

# French months map
fr_mname2mon = dict((m, i + 1) for i, m in enumerate(FR_MONTHS) if m)
frlc_mname2mon = dict((m, i + 1) for i, m in enumerate(FR_MONTHS_LC) if m)

BASE_PATTERNS_FR = {
    "pat:fr:months": oneOf(FR_MONTHS).setParseAction(lambda t: fr_mname2mon[t[0]]),
    "pat:fr:months_lc": oneOf(FR_MONTHS_LC).setParseAction(
        lambda t: frlc_mname2mon[t[0]]
    ),
}

PATTERNS_FR = [
    # French patterns
    {
        "key": "dt:date:fr_base",
        "name": "Base french date with month name not archive",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_FR["pat:fr:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:fr_base_lc",
        "name": "Base french date with month name and lowcase, no article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_FR["pat:fr:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:fr_base_article",
        "name": "Base french date with month name and articles",
        "pattern": CaselessLiteral("Le").suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_FR["pat:fr:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:fr_base_lc_article",
        "name": "Base french date with month name and articles and lowcase",
        "pattern": CaselessLiteral("le").suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_FR["pat:fr:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
]



# -*- coding: utf-8 -*-                6
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


EN_NUMERIC_SUFFIXES = ["nd", "rd", "th", "st"]
PAT_EN_DAY_NUMERIC = (
    Word(nums, min=1, max=2).setResultsName("day")
    + oneOf(EN_NUMERIC_SUFFIXES).suppress()
)

BASE_DATE_PATTERNS = {
    "pat:date:d.m": Word(nums, exact=2) + Literal(".").suppress() + Word(nums, exact=2),
    "pat:date:d/m/yyyy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal("/").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("/").suppress()
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:m/d/yy": Word(nums, min=1, max=2).setResultsName("month")
    + Literal("/").suppress()
    + Word(nums, min=1, max=2).setResultsName("day")
    + Literal("/").suppress()
    + Word(nums, exact=2).setResultsName("year"),
    "pat:date:d/m/yy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal("/").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("/").suppress()
    + Word(nums, exact=2).setResultsName("year"),
    "pat:date:d.m.yyyy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal(".").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal(".").suppress()
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:yyyy/m/d": Word(nums, exact=4).setResultsName("year")
    + "/"
    + Word(nums, min=1, max=2).setResultsName("month")
    + "/"
    + Word(nums, min=1, max=2).setResultsName("day"),
    "pat:date:d.m.yy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal(".").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal(".").suppress()
    + Word(nums, exact=2).setResultsName("year"),
    "pat:date:d-m-yy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal("-").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("-").suppress()
    + Word(nums, exact=2).setResultsName("year"),
    "pat:date:d-m-yyyy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal("-").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("-").suppress()
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:yyyy-m-d": Word(nums, exact=4).setResultsName("year")
    + Literal("-").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("-").suppress()
    + Word(nums, min=1, max=2).setResultsName("day"),
    "pat:date:yyyy.m.d": Word(nums, exact=4).setResultsName("year")
    + Literal(".").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal(".").suppress()
    + Word(nums, min=1, max=2).setResultsName("day"),
    "pat:date:ddmmyyyy": Word(nums, exact=2).setResultsName("day")
    + Word(nums, exact=2).setResultsName("month")
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:mmyyyy": Word(nums, exact=2).setResultsName("month")
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:ddmmyyyy": Word(nums, exact=2).setResultsName("day")
    + Word(nums, exact=2).setResultsName("month")
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:mmyyyy": Word(nums, exact=2).setResultsName("month")
    + Word(nums, exact=4).setResultsName("year"),
    "pat:date:yyyymmdd": Word(nums, exact=4).setResultsName("year")
    + Word(nums, exact=2).setResultsName("month")
    + Word(nums, exact=2).setResultsName("day"),
    "pat:date:mm/dd/yyyy": Word(nums, min=1, max=2).setResultsName("month")
    + Literal("/").suppress()
    + Word(nums, min=1, max=2).setResultsName("day")
    + Literal("/").suppress()
    + Word(nums, exact=4).setResultsName("year"),
    # Rare patterns
    "pat:date:d/m yy": Word(nums, min=1, max=2).setResultsName("day")
    + Literal("/").suppress()
    + Word(nums, min=1, max=2).setResultsName("month")
    + Literal("‘").suppress()
    + Word(nums, exact=2).setResultsName("year"),
}

BASE_TIME_PATTERNS = {
    "pat:time:minutes": Word(nums, exact=2).setResultsName("hour")
    + Literal(":").suppress()
    + Word(nums, exact=2).setResultsName("minute"),
    "pat:time:full": Word(nums, exact=2).setResultsName("hour")
    + Literal(":").suppress()
    + Word(nums, exact=2).setResultsName("minute")
    + Literal(":").suppress()
    + Word(nums, exact=2).setResultsName("second") + Optional(Literal('+').suppress() + Word(nums, min=3, max=4).setResultsName('timezone')),
}


ENG_MONTHS = [
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
]
ENG_MONTHS_LC = [
    "january",
    "february",
    "march",
    "april",
    "may",
    "june",
    "jule",
    "august",
    "september",
    "october",
    "november",
    "december",
]
ENG_MONTHS_SHORT = [
    "jan",
    "feb",
    "mar",
    "apr",
    "may",
    "jun",
    "jul",
    "aug",
    "sep",
    "oct",
    "nov",
    "dec",
]
ENG_WEEKDAYS = [
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Satuday",
    "Sunday",
]
ENG_WEEKDAYS_SHORT = ["mon", "tue", "wed", "thu", "fri", "sat", "sun"]


# English months map
en_mname2mon = dict((m, i + 1) for i, m in enumerate(ENG_MONTHS) if m)
ensh_mname2mon = dict((m, i + 1) for i, m in enumerate(ENG_MONTHS_SHORT) if m)
enlc_mname2mon = dict((m, i + 1) for i, m in enumerate(ENG_MONTHS_LC) if m)
ensh_wday2weekday = dict((m, i + 1) for i, m in enumerate(ENG_WEEKDAYS_SHORT) if m)

BASE_PATTERNS_EN = {
    "pat:eng:months": oneOf(ENG_MONTHS).setParseAction(lambda t: en_mname2mon[t[0]]),
    "pat:eng:months:lc": oneOf(ENG_MONTHS_LC).setParseAction(
        lambda t: enlc_mname2mon[t[0]]
    ),
    "pat:eng:months:short": oneOf(ENG_MONTHS_SHORT, caseless=True).setParseAction(
        lambda t: ensh_mname2mon[t[0].lower()]
    ),
    "pat:eng:weekdays": oneOf(ENG_WEEKDAYS),
    "pat:eng:weekdays:short": oneOf(ENG_WEEKDAYS_SHORT, caseless=True).setParseAction(
        lambda t: ensh_wday2weekday[t[0].lower()]
    ),
}


PATTERNS_EN = [
    # Universal patterns
    {
        "key": "dt:date:date_1",
        "name": "Datetime string",
        "pattern": BASE_DATE_PATTERNS["pat:date:d/m/yyyy"],
        "length": {"min": 8, "max": 10},
        "format": "%d/%m/%Y",
    },
    {
        "key": "dt:date:date_2",
        "name": "Datetime string",
        "pattern": BASE_DATE_PATTERNS["pat:date:d.m.yyyy"],
        "length": {"min": 8, "max": 10},
        "format": "%d.%m.%Y",
    },
    {
        "key": "dt:date:date_3",
        "name": "Datetime string",
        "pattern": BASE_DATE_PATTERNS["pat:date:yyyy/m/d"],
        "length": {"min": 8, "max": 10},
        "format": "%Y/%m/%d",
    },
    {
        "key": "dt:date:date_4",
        "name": "Datetime string",
        "pattern": BASE_DATE_PATTERNS["pat:date:d.m.yy"],
        "length": {"min": 6, "max": 8},
        "format": "%d.%m.%y",
        "yearshort": True,
    },
    {
        "key": "dt:date:date_iso8601",
        "name": "ISO 8601 date",
        "pattern": BASE_DATE_PATTERNS["pat:date:d-m-yyyy"],
        "length": {"min": 8, "max": 10},
        "format": "%d-%m-%Y",
    },
    {
        "key": "dt:date:date_iso8601_short",
        "name": "ISO 8601 date shorted",
        "pattern": BASE_DATE_PATTERNS["pat:date:d-m-yy"],
        "length": {"min": 6, "max": 8},
        "format": "%d-%m-%Y",
        "yearshort": True,
    },
# Commented since it's very rare and generates too many false positives
#    {
#        "key": "dt:date:date_7",
#        "name": "Year-month string",
#        "pattern": BASE_DATE_PATTERNS["pat:date:mmyyyy"],
#        "length": {"min": 6, "max": 6},
#        "format": "%m.%Y",
#    },
    {
        "key": "dt:date:date_8",
        "name": "Date with 2-digits year",
        "pattern": BASE_DATE_PATTERNS["pat:date:d/m/yy"],
        "length": {"min": 6, "max": 8},
        "format": "%d/%m/%y",
        "yearshort": True,
    },
    {
        "key": "dt:date:date_9",
        "name": "Date as ISO",
        "pattern": BASE_DATE_PATTERNS["pat:date:yyyy-m-d"],
        "length": {"min": 6, "max": 10},
        "format": "%Y-%m-%d",
    },
    {
        "key": "dt:date:date_10",
        "name": "Date as yyyy.mm.dd",
        "pattern": BASE_DATE_PATTERNS["pat:date:yyyy.m.d"],
        "length": {"min": 6, "max": 10},
        "format": "%Y.%m.%d",
    },
    # USA patterns
    {
        "key": "dt:date:date_usa_1",
        "name": "Date with 2-digits year",
        "pattern": BASE_DATE_PATTERNS["pat:date:m/d/yy"],
        "length": {"min": 6, "max": 8},
        "format": "%m/%d/%y",
        "yearshort": True,
    },
    {
        "key": "dt:date:date_usa",
        "name": "USA mm/dd/yyyy string",
        "pattern": BASE_DATE_PATTERNS["pat:date:mm/dd/yyyy"],
        "length": {"min": 8, "max": 10},
        "format": "%m/%d/%Y",
    },
    # English patterns
    {
        "key": "dt:date:date_eng1",
        "name": "Date with english month and possible dots",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_EN["pat:eng:months"].setResultsName("month")
        + Optional(".").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%d.%b.%Y",
    },
    {
        "key": "dt:date:date_eng1x",
        "name": "Date with english month and , ",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_EN["pat:eng:months"].setResultsName("month")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%d.%b.%Y",
    },
    {
        "key": "dt:date:date_eng1_lc",
        "name": "Date with english month lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_EN["pat:eng:months:lc"].setResultsName("month")
        + Optional(".").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%d.%b.%Y",
    },
    {
        "key": "dt:date:date_eng1_short",
        "name": "Date with english month short",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional(".").suppress()
        + BASE_PATTERNS_EN["pat:eng:months:short"].setResultsName("month")
        + Optional(".").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 10},
        "format": "%d.%b.%Y",
    },
    {
        "key": "dt:date:date_eng2",
        "name": "Date with english month 2",
        "pattern": BASE_PATTERNS_EN["pat:eng:months"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%b %d, %Y",
    },
    {
        "key": "dt:date:date_eng2_lc",
        "name": "Date with english month 2 lowcase",
        "pattern": BASE_PATTERNS_EN["pat:eng:months:lc"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%b %d, %Y",
    },
    {
        "key": "dt:date:date_eng2_short",
        "name": "Date with english month 2 short",
        "pattern": BASE_PATTERNS_EN["pat:eng:months:short"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 10},
        "format": "%b %d, %Y",
    },
    {
        "key": "dt:date:date_eng3",
        "name": "Date with english month full",
        "pattern": BASE_PATTERNS_EN["pat:eng:months:lc"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Optional(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 10, "max": 20},
        "format": "%b %d, %Y",
        "filter": 2,
    },
    {
        "key": "dt:date:date_eng4_short",
        "name": "Date with english month short with dash",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + Optional("-").suppress()
        + BASE_PATTERNS_EN["pat:eng:months:short"].setResultsName("month")
        + Optional("-").suppress()
        + Word(nums, exact=2).setResultsName("year"),
        "length": {"min": 10, "max": 10},
        "format": "%d-%b-%y",
        "yearshort": True,
    },
    {
        "key": "dt:date:noyear_1",
        "name": "Datetime string without year",
        "pattern": BASE_DATE_PATTERNS["pat:date:d.m"],
        "length": {"min": 5, "max": 5},
        "format": "%d.%m",
        "noyear": True,
    },
    {
        "key": "dt:date:date_4_point",
        "name": "Datetime string",
        "pattern": BASE_DATE_PATTERNS["pat:date:d.m.yy"] + Literal(".").suppress(),
        "length": {"min": 6, "max": 9},
        "format": "%d.%m.%y",
        "yearshort": True,
    },
    {
        "key": "dt:date:weekday_eng",
        "name": "Date with english month and weekday",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays"].suppress()
        + Optional(",").suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_EN["pat:eng:months"].setResultsName("month")
        + Optional(Literal(",")).suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 17, "max": 27},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_eng_lc",
        "name": "Date with english month and weekday",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays"].suppress()
        + Optional(",").suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_EN["pat:eng:months:lc"].setResultsName("month")
        + Optional(Literal(",")).suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 17, "max": 27},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_eng_wshort",
        "name": "Date with english month and weekday",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays:short"].suppress()
        + Optional(",").suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_EN["pat:eng:months"].setResultsName("month")
        + Optional(Literal(",")).suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 16, "max": 27},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_eng_mshort_wshort",
        "name": "Date with short english month and short weekday",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays:short"].suppress()
        + Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_EN["pat:eng:months:short"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 15, "max": 15},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_eng_iso",
        "name": "Date with english weekday and iso date",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays"].suppress()
        + Optional(",").suppress()
        + BASE_DATE_PATTERNS["pat:date:d/m/yyyy"],
        "length": {"min": 13, "max": 25},
        "format": "%d/%m/%Y",
        "filter": 1,
    },
    {
        "key": "dt:date:weekday_short_eng_iso",
        "name": "Date with english short weekday and iso date",
        "pattern": BASE_PATTERNS_EN["pat:eng:weekdays:short"].suppress()
        + Optional(",").suppress()
        + BASE_DATE_PATTERNS["pat:date:d/m/yyyy"],
        "length": {"min": 13, "max": 18},
        "format": "%d/%m/%Y",
        "filter": 1,
    },
]

INTEGER_LIKE_PATTERNS = [
    {
        "key": "dt:date:date_5",
        "name": "Datetime string as ddmmyyyy",
        "pattern": BASE_DATE_PATTERNS["pat:date:ddmmyyyy"],
        "length": {"min": 8, "max": 8},
        "format": "%d%m%Y",
    },
    {
        "key": "dt:date:date_6",
        "name": "Datetime string as yyyymmdd",
        "pattern": BASE_DATE_PATTERNS["pat:date:yyyymmdd"],
        "length": {"min": 8, "max": 8},
        "format": "%Y%m%d",
    },
]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)

PT_MONTHS = [
    "Janeiro",
    "Fevereiro",
    "Março",
    "Abril",
    "Maio",
    "Junho",
    "Julho",
    "Agosto",
    "Setembro",
    "Outubro",
    "Novembro",
    "Dezembro",
]
PT_MONTHS_LC = [
    "janeiro",
    "fevereiro",
    "março",
    "abril",
    "maio",
    "junho",
    "julho",
    "agosto",
    "setembro",
    "outubro",
    "novembro",
    "dezembro",
]

# Portugal months map
pt_mname2mon = dict((m, i + 1) for i, m in enumerate(PT_MONTHS) if m)
ptlc_mname2mon = dict((m, i + 1) for i, m in enumerate(PT_MONTHS_LC) if m)

BASE_PATTERNS_PT = {
    "pat:pt:months": oneOf(PT_MONTHS).setParseAction(lambda t: pt_mname2mon[t[0]]),
    "pat:pt:months_lc": oneOf(PT_MONTHS_LC).setParseAction(
        lambda t: ptlc_mname2mon[t[0]]
    ),
}

PATTERNS_PT = [
    # Portugal patterns
    {
        "key": "dt:date:pt_base",
        "name": "Base portugal date with month name not article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_PT["pat:pt:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:pt_base_lc",
        "name": "Base portugal date with month name and lowcase, no article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_PT["pat:pt:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:pt_base_article",
        "name": "Base portugal date with month name and articles",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_PT["pat:pt:months"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:pt_base_lc_article",
        "name": "Base portugal date with month name and articles and lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_PT["pat:pt:months_lc"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


IT_WEEKDAYS = [
    "Lunedì",
    "Martedì",
    "Mercoledì",
    "Giovedì",
    "Venerdì",
    "Sabato",
    "Domenica",
]
IT_WEEKDAYS_LC = [
    "lunedì",
    "martedì",
    "mercoledì",
    "giovedì",
    "venerdì",
    "sabato",
    "domenica",
]


IT_MONTHS = [
    "Gennaio",
    "Febbraio",
    "Marzo",
    "Aprile",
    "Maggio",
    "Giugno",
    "Luglio",
    "Agosto",
    "Settembre",
    "Ottobre",
    "Novembre",
    "Dicembre",
]
IT_MONTHS_LC = [
    "gennaio",
    "febbraio",
    "marzo",
    "aprile",
    "maggio",
    "giugno",
    "luglio",
    "agosto",
    "settembre",
    "ottobre",
    "novembre",
    "dicembre",
]

# Spanish months map
it_mname2mon = dict((m, i + 1) for i, m in enumerate(IT_MONTHS) if m)
itlc_mname2mon = dict((m, i + 1) for i, m in enumerate(IT_MONTHS_LC) if m)

BASE_PATTERNS_IT = {
    "pat:it:months": oneOf(IT_MONTHS).setParseAction(lambda t: it_mname2mon[t[0]]),
    "pat:it:months_lc": oneOf(IT_MONTHS_LC).setParseAction(
        lambda t: itlc_mname2mon[t[0]]
    ),
}


PATTERNS_IT = [
    # Italian date patterns
    {
        "key": "dt:date:it_base",
        "name": "Base italian date with month name not article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_IT["pat:it:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:it_base_lc",
        "name": "Base italian date with month name and lowcase, no article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_IT["pat:it:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:it_base_article",
        "name": "Base italian date with month name and articles",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_IT["pat:it:months"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:it_base_lc_article",
        "name": "Base italian date with month name and articles and lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_IT["pat:it:months_lc"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:it_rare_1",
        "name": "Italian date stars with month name",
        "pattern": BASE_PATTERNS_IT["pat:it:months"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 25},
        "format": "%M %d %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:it_rare_2",
        "name": "Italian date stars with month name lowcase",
        "pattern": BASE_PATTERNS_IT["pat:it:months_lc"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 25},
        "format": "%M %d %Y",
        "filter": 1,
    },
]



# -*- coding: utf-8 -*-
from pyparsing import (
    Word,
    nums,
    alphas,
    oneOf,
    lineStart,
    lineEnd,
    Optional,
    restOfLine,
    Literal,
    ParseException,
    CaselessLiteral,
)


ES_MONTHS = [
    "Enero",
    "Febrero",
    "Marzo",
    "Abril",
    "Mayo",
    "Junio",
    "Julio",
    "Agosto",
    "Septiembre",
    "Octubre",
    "Noviembre",
    "Diciembre",
]
ES_MONTHS_LC = [
    "enero",
    "febrero",
    "marzo",
    "abril",
    "mayo",
    "junio",
    "julio",
    "agosto",
    "septiembre",
    "octubre",
    "noviembre",
    "diciembre",
]

# Spanish months map
es_mname2mon = dict((m, i + 1) for i, m in enumerate(ES_MONTHS) if m)
eslc_mname2mon = dict((m, i + 1) for i, m in enumerate(ES_MONTHS_LC) if m)

BASE_PATTERNS_ES = {
    "pat:es:months": oneOf(ES_MONTHS).setParseAction(lambda t: es_mname2mon[t[0]]),
    "pat:es:months_lc": oneOf(ES_MONTHS_LC).setParseAction(
        lambda t: eslc_mname2mon[t[0]]
    ),
}


PATTERNS_ES = [
    # Spanish patterns
    {
        "key": "dt:date:es_base",
        "name": "Base spanish date with month name not article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_ES["pat:es:months"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:es_base_lc",
        "name": "Base spanish date with month name and lowcase, no article",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + BASE_PATTERNS_ES["pat:es:months_lc"].setResultsName("month")
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 22},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:es_base_article",
        "name": "Base spanish date with month name and articles",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_ES["pat:es:months"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:es_base_lc_article",
        "name": "Base spanish date with month name and articles and lowcase",
        "pattern": Word(nums, min=1, max=2).setResultsName("day")
        + CaselessLiteral("de").suppress()
        + BASE_PATTERNS_ES["pat:es:months_lc"].setResultsName("month")
        + CaselessLiteral("de").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 26},
        "format": "%d %m %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:es_rare_1",
        "name": "Spanish date stars with month name",
        "pattern": BASE_PATTERNS_ES["pat:es:months"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 25},
        "format": "%M %d %Y",
        "filter": 1,
    },
    {
        "key": "dt:date:es_rare_2",
        "name": "Spanish date stars with month name lowcase",
        "pattern": BASE_PATTERNS_ES["pat:es:months_lc"].setResultsName("month")
        + Word(nums, min=1, max=2).setResultsName("day")
        + Literal(",").suppress()
        + Word(nums, exact=4).setResultsName("year"),
        "length": {"min": 11, "max": 25},
        "format": "%M %d %Y",
        "filter": 1,
    },
]



import re
import io
from setuptools import setup, find_packages

open_as_utf = lambda x: io.open(x, encoding='utf-8')

(__version__, ) = re.findall("__version__.*\s*=\s*[']([^']+)[']", open('qddate/__init__.py').read())

readme = re.sub(r':members:.+|..\sautomodule::.+|:class:|:func:', '', open_as_utf('README.rst').read())
readme = re.sub(r'`Settings`_', '`Settings`', readme)
readme = re.sub(r'`Contributing`_', '`Contributing`', readme)
readme = open_as_utf('README.rst').read()
history = re.sub(r':mod:|:class:|:func:', '', open_as_utf('HISTORY.rst').read())



setup(
    name='qddate',
    version=__version__,
    description='Quick and dirty date parsing Python library to parse HTML dates really fast',
    long_description=readme + '\n\n\n' + history,
    author='Ivan Begtin',
    author_email='ivan@begtin.tech',
    url='https://github.com/ivbeg/qddate',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    install_requires=[
        'dateparser>=1.1.0',
        'pyparsing>=2.4.7'
    ],
    python_requires='>=3.8',
    license="BSD",
    zip_safe=False,
    keywords='date datetime',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Ivan Begtin (ivan@begtin.tech)"
__license__ = "BSD"


from .patterns import ALL_PATTERNS, BASE_TIME_PATTERNS
from pyparsing import Optional, lineStart, oneOf, Literal, restOfLine, ParseException
import datetime
from .dirty import matchPrefix


class DateParser:
    """Class to use pyparsing-based patterns to parse dates"""

    def __init__(self, generate=True, patterns=ALL_PATTERNS, base_only=False):
        """Inits class DataParser
        :param generate: Boolean value, if true, than automatically generate all patterns from base list self.patterns
        :param patterns: list of patterns to be used. Default ALL_PATTERNS. See qddate.patterns for more info
        :param base_only: Use only base patterns during generation of final list. Filters all patterns with text after datetime.
        """
        self.patterns = patterns
        if generate:
            self.__generate(base_only)
        self.cachedpats = None
        self.ind = []

    def __matchPrefix(self, text):
        """
        This is silver bullet, cornerstone and magic wand of speed of this algorithm
        it filters patterns using manually selected rules. Yes, yes, it's "dirty" code and it could be beautified in
        many ways but this library intended to be fast, not beautiful. Without matching is x1.3 slower so let it be.
        :param text: text with date to match
        :return: list of patterns to run against
        """
        return matchPrefix(text)

    def startSession(self, cached_p):
        self.cachedpats = [x for x in self.patterns if x["key"] in cached_p]

    def endSession(self):
        self.cachedpats = None

    def __generate(self, base_only=False):
        """Generates dates patterns"""
        base = []
        texted = []
        for pat in self.patterns:
            data = pat.copy()
            data["basekey"] = data["key"]
            data["key"] += ":time_1"
            data["right"] = True
            data["pattern"] = (
                data["pattern"]
                + Optional(Literal(",")).suppress()
                + BASE_TIME_PATTERNS["pat:time:minutes"]
            )
            data["time_format"] = "%H:%M"
            data["length"] = {
                "min": data["length"]["min"] + 5,
                "max": data["length"]["max"] + 8,
            }
            base.append(data)

            data = pat.copy()
            data["basekey"] = data["key"]
            data["right"] = True
            data["key"] += ":time_2"
            data["pattern"] = (
                data["pattern"]
                + Optional(oneOf([",", "|", 'T'])).suppress()
                + BASE_TIME_PATTERNS["pat:time:full"]
            )
            data["time_format"] = "%H:%M:%S"
            data["length"] = {
                "min": data["length"]["min"] + 9,
                "max": data["length"]["max"] + 9,
            }
            base.append(data)

            data = pat.copy()
            data["basekey"] = data["key"]
            data["right"] = True
            data["key"] += ":time_3"
            data["pattern"] = (
                data["pattern"]
                + Optional(Literal("[")).suppress()
                + BASE_TIME_PATTERNS["pat:time:minutes"]
                + Optional(Literal("]")).suppress()
            )
            data["time_format"] = "%H:%M"
            data["length"] = {
                "min": data["length"]["min"] + 7,
                "max": data["length"]["max"] + 10,
            }
            base.append(data)

            data = pat.copy()
            data["pattern"] = data["pattern"]
            data["right"] = True
            data["basekey"] = data["key"]
            base.append(data)

        if not base_only:
            for pat in base:
                # Right
                data = pat.copy()
                data["key"] += ":t_right"
                data["pattern"] = (
                    lineStart
                    + data["pattern"]
                    + Optional(oneOf([",", "|", ":", ")"])).suppress()
                    + restOfLine.suppress()
                )
                data["length"] = {
                    "min": data["length"]["min"] + 1,
                    "max": data["length"]["max"] + 90,
                }
                texted.append(data)

            base.extend(texted)
        self.patterns = base

    def match(self, text, noprefix=False, noyear=True):
        """Matches date/datetime string against date patterns and returns pattern and parsed date if matched.
        It's not indeded for common usage, since if successful it returns date as array of numbers and pattern
        that matched this date

        :param text:
            Any human readable string
        :type date_string: str|unicode
        :param noprefix:
            If set True than doesn't use prefix based date patterns filtering settings
        :type noprefix: bool
        :param noyear:
            If set True than does use patterns with noyear flag (does a lot of false positives) if set False doesn't use patterns with noyear flag
        :type noprefix: bool


        :return: Returns dicts with `values` as array of representing parsed date and 'pattern' with info about matched pattern if successful, else returns None
        :rtype: :class:`dict`."""
        n = len(text)
        if self.cachedpats is not None:
            pats = self.cachedpats
        else:
            pats = self.patterns
        if n > 5 and not noprefix:
            basekeys = self.__matchPrefix(text[:6])
        else:
            basekeys = []
        for p in pats:
            if n < p["length"]["min"] or n > p["length"]["max"]:
                continue
            if p["right"] and len(basekeys) > 0 and p["basekey"] not in basekeys:
                continue
            if not noyear and 'noyear' in p.keys() and p['noyear']:
                continue
            try:
                r = p["pattern"].parseString(text)
                # Do sanity check
                d = r.asDict()
                if "month" in d:
                    val = int(d["month"])
                    if val > 12 or val < 1:
                        continue
                if "day" in d:
                    val = int(d["day"])
                    if val > 31 or val < 1:
                        continue
                return {"values": r, "pattern": p}
            except ParseException as e:
                #                print p['key'], text.encode('utf-8'), e
                pass
        return None

    def parse(self, text, noprefix=False):
        """Parse date and time from given date string.

        :param text:
            Any human readable string
        :type date_string: str|unicode
        :param noprefix:
            If set True than doesn't use prefix based date patterns filtering settings
        :type noprefix: bool


        :return: Returns :class:`datetime <datetime.datetime>` representing parsed date if successful, else returns None
        :rtype: :class:`datetime <datetime.datetime>`."""

        res = self.match(text, noprefix)
        if res:
            r = res["values"]
            p = res["pattern"]
            d = {"month": 0, "day": 0, "year": 0}
            if "noyear" in p and p["noyear"] == True:
                d["year"] = datetime.datetime.now().year
            for k, v in list(r.items()):
                d[k] = int(v)
            dt = datetime.datetime(**d)
            return dt
        return None


if __name__ == "__main__":
    from pprint import pprint

    tests = [
        "01.12.2009",
        "2013-01-12",
        "31.05.2001",
        "7/12/2009",
        "6 Jan 2009",
        "Jan 8, 1098",
        "JAN 1, 2001",
        "3 Января 2003 года",
        "05 Января 2003",
        "12.03.1999 Hello people",
        "15 февраля 2007 года",
        "5 August 2001",
        "3 jun 2009",
        "16 May 2009 14:10",
        "01 february 2009",
        "01.03.2009 14:53",
        "01.03.2009 14:53:12",
        "22.12.2009 17:56",
        "05/16/99",
        "11/29/1991",
        "Thursday 4 April 2019",
        "July 01, 2015",
        "Fri, 3 July 2015",
        u"2 Июня 2015",
        u"9 июля 2015 г.",
        u"26 / 06 ‘15",
        u"09.июля.2015",
        u"14th April 2015:",
        u"23 Jul 2015, 09:00 BST",
        u"пятница, июля 17, 2015",
        u"Июль 16, 2015",
        u"Le 8 juillet 2015",
        u"8 juillet 2015",
        u"Fri 24 Jul 2015",
        u"26 de julho de 2015",
        u"17 de Junio de 2015",
        u"28. Juli 2015",
        u"21 Фeвpyapи 2015",
        u"1 нoeмвpи 2013",
        u"23 июня 2015",
        u"3 Июля, 2015",
        u"7 August, 2015",
        u"Wednesday 22 Apr 2015",
        u"12-08-2015 - 09:00",
        u"08 Jul, 2015",
        u"August 10th, 2015",
        u"junio 9, 2015",
        u"Авг 11, 2015",
        u"Вторник, 18 Август 2015 18:51",
        u"Июль 16th, 2012 | 11:08 пп",
        u"19 август в 16:03",
        u"7 August, 2015",
        u"9 Июля 2015 [11:23]",
    ]

    #    print list(calendar.month_abbr)[1:]
    ind = DateParser(generate=True)
    #    for i in ind.patterns:
    #        print i
    print(len(ind.patterns))
    for text in tests:
        res = ind.match(text)
        print(r)
        if r:
            r = res["values"]
            p = res["pattern"]
            d = {"month": 0, "day": 0, "year": 0}
            if "noyear" in p and p["noyear"] == True:
                d["year"] = datetime.datetime.now().year
            for k, v in list(r.items()):
                d[k] = int(v)
            dt = datetime.datetime(**d)
        else:
            pass

    #    for p in ind.patterns:
    #        pprint(p)
    import dateparser

    for text in tests:
        pass
        # print(dateparser.parse(text))
#    ind.patterns = DATE_DATA_TYPES_RAW
